@@ -14,9 +14,15 @@
 #' @keywords internal
 #'
 energyloss <- function(x0, x, xp, verbose = FALSE) {
-  s1 <- torch_mean(torch_norm(x - x0, 2, dim = 2)) / 2 + torch_mean(torch_norm(xp - x0, 2, dim = 2)) / 2
-  s2 <- torch_mean(torch_norm(x - xp, 2, dim = 2))
-  if (verbose) {
+  if (!requireNamespace("torch", quietly = TRUE)) {
+    stop("This function requires the 'torch' package. Install it with install.packages('torch').", call. = FALSE)
+  }
+
+  s1 <- torch::torch_mean(torch::torch_norm(x - x0, 2, dim = 2)) / 2 +
+    torch::torch_mean(torch::torch_norm(xp - x0, 2, dim = 2)) / 2
+  s2 <- torch::torch_mean(torch::torch_norm(x - xp, 2, dim = 2))
+
+  if (isTRUE(verbose)) {
     return(c((s1 - s2 / 2), s1, s2))
   } else {
     return(s1 - s2 / 2)
